<?php
namespace PME\Modules\Planejamento\Prato;

use Zeedhi\Framework\DTO;

class EmissaoReceitasController extends \Zeedhi\Framework\Controller\Report {
    protected $reportService;
    protected $birtStrategy;
    protected $utilities;
    protected $Lista;
    protected $crudService;

    public function __construct($reportService, $birtStrategy, $utilities, $Lista, $crudService) {
        $this->birtStrategy  = $birtStrategy;
        $this->utilities     = $utilities;
        $this->Lista         = $Lista;
        $this->crudService    = $crudService;

        parent::__construct($reportService);
    }
    
    protected $reportMapping = array (
        'PLA01300' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "CDFILIAL"     => "P_CDFILIAL",
                "PRATOINI"     => "P_CDPRATOINICIO",
                "PRATOFIM"     => "P_CDPRATOFINAL",
                "NIVEL"        => "P_NIVEL",
                "MAXNIVEL"     => "P_MAXNIVEL",
                "CDFILIPROC"   => "P_CDFILIPROC",
                "NRORG"        => "P_NRORG",
                "CDFILCENTPME" => "P_CDFILCENTPME"
            )
        ),
        'pla01305' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "LISTAFILI"  =>  "P_LISTAFILI",
                "PRATOINI"   =>  "P_CDPRATOINIC",
                "PRATOFIM"   =>  "P_CDPRATOFINA",
                "DTINI"      =>  "P_DTINICIAL",
                "DTFIN"      =>  "P_DTFINAL",
                "PRATOINI"   =>  "P_PRATOINICIAL",
                "PRATOFIM"   =>  "P_PRATOFINAL"
            )
        ),
        'pla01306' => array (
            'strategy' => 'BIRT',
            'parameterMapping'   => array (
                "CDFILIAL"     => "P_CDFILIAL",
                "PRATOINI"     => "P_CDPRATOINIC",
                "PRATOFIM"     => "P_CDPRATOFINA",
                "CDPICTPRAT"   => "P_CDPICTPRAT",
                "NRORG"        => "P_NRORG",
                "NRORG1"       => "NRORG",
                "CDFILCENTPME" => "P_CDFILCENTPME"
            )
        ),
        'pla05500' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "IDTIPORECE"  => "P_IDTIPORECE",
                "CDFILIALAUX" => "P_CDFILIAL",
                "CDFILCENTPME"=> "P_CDFILCENTPME",
                "CDTIPOCARD"  => "P_CDTIPOCARD",
                "PRODINICIAL" => "P_PRODUTOINI",
                "PRODFINAL"   => "P_PRODUTOFIN",
                "CDPICTPROD"  => "P_CDPICTPROD",
                "CDPICTPRAT"  => "P_CDPICTPRAT",
                "LISTASERV"   => "P_LISTASERV",
                "LISTAFILI"   => "P_LISTAFILI",
                "NRORG"       => "P_NRORG"
            )
        ),
        'pla05000' => array (
            'strategy' => 'BIRT',
            'parameterMapping'   => array (
                "IDTIPORECE"   => "P_IDTIPORECE",
                "CDFILIAL"     => "P_CDFILIAL",
                "CDNUTRIENTE"  => "P_CDNUTRIENTE",
                "CDFILIALAUX"  => "P_CDFILAUXPRAT",
                "CDSERVFILI"   => "P_CDSERVFILI",
                "CDFILCENTPME" => "P_CDFILCENTPME",
                "CDTIPOCARD"   => "P_CDTIPOCARD",
                "NIVEL"        => "P_NIVEL",
                "PRATOINI"     => "P_CDPRATOINIC",
                "PRATOFIM"     => "P_CDPRATOFINA",
                "ORDENA"       => "P_ORDENA",
                "CDPICTPRAT"   => "P_CDPICTPRAT",
                "NRORG"        => "P_NRORG"
            )
        ),
        'pla05400' => array (
            'strategy' => 'BIRT',
            'parameterMapping'   => array (
                "GRUPFILI"     => "P_CDGRUPO",
                "CDFILIAL"     => "P_CDFILIAL",
                "CDFILIALAUX"  => "P_CDFILAUXPRAT",
                "PRATOINI"     => "P_CDPRATINI",
                "PRATOFIM"     => "P_CDPRATFIN",
                "PRODINICIAL"  => "P_CDPRODINI",
                "PRODFINAL"    => "P_CDPRODFIN",
                "IDTPREQPRODFILI" => "P_IDTPREQPRODFILI",
                "CDPICTPRAT"   => "P_CDPICTPRAT",
                "CDPICTPROD"   => "P_CDPICTPROD",
                "CDFILCENTPME"   => "P_CDFILCENTPME",
                "NRORG"        => "P_NRORG"
            )
        ),
        'pla013B' => array (
            'strategy' => 'BIRT',
            'parameterMapping'   => array (
                "IDTIPORECE"   => "P_IDTIPORECE",
                "CDFILIAL"     => "P_CDFILIAL",
                "CDSERVFILI"   => "P_CDSERVFILI",
                "PRATOINI"     => "P_CDPRATOINIC",
                "PRATOFIM"     => "P_CDPRATOFINA",
                "CDPICTPRAT"   => "P_CDPICTPRAT",
                "CDPICTPROD"   => "P_CDPICTPROD",
                "NRORG"        => "P_NRORG",
                "CDNUTRIENTE"  => "P_CDNUTRIENTE"
            )
        ),
    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrorg = $this->utilities->getNrOrg();
        $row['NRORG'] = $nrorg;
        $row['NRORG1'] = $nrorg;
        
        //NIVEL
        if($row['ORDENA'] == 0){
            $row['NIVEL'] = 10;
        }else{
            $row['NIVEL'] = array_key_exists('NIVEL', $row) ? $row['NIVEL'] * 2 : 0;
        }

        $row['PRATOFIM'] = $row['PRATOFIM'] . "Z";    
        $row['PRODINICIAL'] = $row['PRODINI'];
        $row['PRODFINAL'] = $row['PRODFIN'] . "Z";

        if ($row['CD_REPORT_CODE'] === 'pla05500') {
            if ($row['IDTIPORECE'] === 'T') {
                $row['CDFILIAL'] = $row['CDFILCENTPME'];
                $row['LISTAFILI'] = '';
                $row['LISTASERV'] = '';
            } else {
                if (count($row['CDSERVFILI']) > 0) {
                   $servico = join("_", $row['CDSERVFILI']);
                } else {
                  $servicos = $this->getServicosEmissaoReceitas($row);
                  $servico = '';
                  foreach ($servicos as $serv) {
                      if (strpos($servico, $serv['FILISERV']) === false) {
                        $servico .= $serv['FILISERV'] . "_";
                      }
                  }
                }
                $row['LISTASERV'] = $this->Lista->geraLista($servico, $nrorg);
                
                if ($row['IDTIPORECE'] === 'P') {
                    $unidade = $row['CDFILIAL2'];
                    $row['LISTAFILI'] = $this->Lista->geraLista($unidade, $nrorg);
                    $row['CDFILIALAUX'] = $unidade;
                } 
                else {
                    if (count($row['CDFILIAL']) > 0) {
                      $unidade = join("_", $row['CDFILIAL']);
                    } else {
                      $unidades = $this->getUnidadesEmissaoReceitas($row);
                      $unidade = '';
                      foreach ($unidades as $uni) {
                          if (strpos($unidade, $uni['CDFILIAL']) === false) {
                            $unidade .= $uni['CDFILIAL'] . "_";
                          }
                      }
                    } 
                    $row['LISTAFILI'] = $this->Lista->geraLista($unidade, $nrorg);
                }
                
                if (count($row['CDTIPOCARD1']) > 0) {
                  $tpCard = join("_", $row['CDTIPOCARD1']);
                } else {
                    $row['CDTIPOCARD'] = $row['CDTIPOCARD1'];
                    $tpCards = $this->getTpcardEmissaoReceitas($row);           
                    $tpCard = '';               
                    foreach ($tpCards as $tpc) {
                      if (strpos($tpCard, $tpc['CDTIPOCARD']) === false) {
                        $tpCard .= $tpc['CDTIPOCARD'] . "_";
                      }
                    }
                }
                $row['CDTIPOCARD'] = $this->Lista->geraLista($tpCard, $nrorg);
            }
            
            if ($row['PROJECT'] === 'PAE' AND $row['IDTIPORECE'] === 'T') {

                if (count($row['CDTIPOCARD1']) > 0) {
                  $tpCard = join("_", $row['CDTIPOCARD1']);
                } else {
                    $row['CDTIPOCARD'] = $row['CDTIPOCARD1'];
                    $tpCards = $this->getTpcardEmissaoReceitas($row);           
                    $tpCard = '';               
                    foreach ($tpCards as $tpc) {
                      if (strpos($tpCard, $tpc['CDTIPOCARD']) === false) {
                        $tpCard .= $tpc['CDTIPOCARD'] . "_";
                      }
                    }
                }
                $row['CDTIPOCARD'] = $this->Lista->geraLista($tpCard, $nrorg);
            } 
        }
        
        if ($row['CD_REPORT_CODE'] === 'pla05000') {
            if ($row['CDSERVFILI1'] === null || $row['CDSERVFILI2'] === null) {
                $row['CDSERVFILI'] = 'X';
            }
            if (count($row['CDNUTRIENTE']) > 0) {
               $row['CDNUTRIENTE'] = join("_", $row['CDNUTRIENTE']);
            } else {
               $row['CDNUTRIENTE'] = 'T';
            }
            
            if ($row['PROJECT'] === 'PAE' AND $row['IDTIPORECE'] === 'T') { 
                if ($row['CDTIPOCARD2']) {
                  $tpCard = $row['CDTIPOCARD2'];
                } else {
                    $row['CDTIPOCARD'] = $row['CDTIPOCARD2'];
                    $tpCards = $this->getTpcardEmissaoReceitas($row);           
                    $tpCard = '';               
                    foreach ($tpCards as $tpc) {
                      if (strpos($tpCard, $tpc['CDTIPOCARD']) === false) {
                        $tpCard .= $tpc['CDTIPOCARD'] . "_";
                      }
                    }
                }
            } else {
                $tpCard = '';
            }
            $row['CDTIPOCARD'] = $tpCard;
        }
        
        if ($row['CD_REPORT_CODE'] === 'pla05400') {
            if (!($row['GRUPFILI'])) {
                $row['GRUPFILI'] = 'T';
            }
            
            if (count($row['CDFILIAL']) > 0) {
                $unidade = join("_", $row['CDFILIAL']);
            } else {
                $unidade = 'T';
            }
            $row['CDFILIAL'] = $unidade;
            
            $rowsReq = $row['IDTPREQPRODFILI'];
            $tipoReqProd = 'EDP';    
            if (!(in_array('E', $rowsReq))) $tipoReqProd = substr_replace($tipoReqProd, 'X', 0, 1);
            if (!(in_array('D', $rowsReq))) $tipoReqProd = substr_replace($tipoReqProd, 'X', 1, 1);
            if (!(in_array('P', $rowsReq))) $tipoReqProd = substr_replace($tipoReqProd, 'X', 2, 1);
            $row['IDTPREQPRODFILI'] = $tipoReqProd;
        }
        
        if ($row['CD_REPORT_CODE'] === 'pla013B') {
            if (count($row['CDNUTRIENTE']) > 0) {
               $row['CDNUTRIENTE'] = join("_", $row['CDNUTRIENTE']);
            } else {
               $row['CDNUTRIENTE'] = 'T';
            }
        }

        $this->birtStrategy->setReportLocale($this->utilities->getLanguage());
        $this->birtStrategy->setReportFormat(strtolower($row['reportFormat']));   
    }
  
    private function getServicosEmissaoReceitas($rows) {
        $dataSourceName = 'servicoAgrupGrupoUnidadeInd';
        
        if (empty($rows['CDFILIAL'])) $cdfilial = 'T';
        else {
          $unidadeRow = [];
          $cdfilial = join("_", $rows['CDFILIAL']);
        }
        
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $cdfilial
            ),
            array(
                'columnName' => 'CDGRUPSERV',
                'operator' => '=',
                'value' => 'T'
            ),
            array(
                'columnName' => 'CDGRUPFILI',
                'operator' => '=',
                'value' => $rows['GRUPFILI'] ? $rows['GRUPFILI'] : 'T'
            ),
            array(
                'columnName' => 'IDTIPOSERV',
                'operator' => '=',
                'value' => $rows['IDTIPOSERV']
            )
        );
                
        $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        return $result;
    }
    
    private function getUnidadesEmissaoReceitas($rows) {
        $dataSourceName = 'unidadeIdcntrresc';
        
        if (empty($rows['CDFILIAL'])) $cdfilial = 'T';
        else {
          $unidadeRow = [];
          $cdfilial = join("_", $rows['CDFILIAL']);
        }
        
        $conditions = array(
             array(
                 'columnName' => 'CDGRUPFILI',
                 'operator' => '=',
                 'value' => $rows['GRUPFILI'] ? $rows['GRUPFILI'] : 'T'
             )
        );
                
        $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        return $result;
    }
    
    private function getTpcardEmissaoReceitas($rows){
        $nrorg = $this->utilities->getNrOrg();
        $dataSourceName = 'tpCardapioPorCentralAtivo';
        
        $conditions = array(       
            array(
                'columnName' => 'IDTPCARDATIV',
                'operator' => '=',
                'value' => 'S'
            ),
            array(
                'columnName' => 'CDFILCENTPME',
                'operator' => '=',
                'value' =>  $rows['CDFILCENTPME']
            ),
            array(
                'columnName' => 'NRPASTATCARD',
                'operator' => '=',
                'value' => null
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $nrorg
            )
        );
        
        $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        return $result;
    }
}