import {
  IDictionary, IEventParam, Loader, Metadata, Datasource, Http, Config,
} from '@zeedhi/core';
import {
  Form, GridEditable, Card, TextInput, DialogService,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Ger28000TipoRetiradaController {
  private utilities = Loader.getInstance('Utilities');
  
  public crudController = Loader.getInstance('CrudController');

  public gridEditController = Loader.getInstance('GridEditableController');
  
  private detailIsVisible = false;
  
  public selectedTiporetirada = '';
  
  public checkedRowsProdutos: IDictionary = {};
  
  get gridColsReturnValue(): number {
    return this.detailIsVisible ? 6 : 12;
  }
  
  public fillCode({ component }: IEventParam<TextInput>) {
    component.value = component.value.padStart(2, '0');
  }
  
  public closeCardDetail() {
    const card = Metadata.getInstance<Card>('detailProdutosCard');
    
    card.isVisible = false;
    this.detailIsVisible = false;
  }
  
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValue: 'CDPRODUTO',
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/find/productList',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
  
  public onChangeTipoRetiradaCurrentRow({ component }: IEventParam<Datasource>) {
    const colunaTipoRet = component.currentRow.CDTIPORETI;
    const gridTipoRetirada: GridEditable = Metadata.getInstance('gridTipoRetirada');
    const isEditing = gridTipoRetirada.editing;
    
    if (colunaTipoRet && !isEditing) {
      const gridProdutos = Metadata.getInstance<GridEditable>('gridProdutos');
      this.selectedTiporetirada = `${component.currentRow.CDTIPORETI} - ${component.currentRow.NMTIPORETI}`;
      
      const conditions = [
        {
          name: 'P_CDTIPORETI',
          operator: '=',
          value: component.currentRow.CDTIPORETI,
        },
      ];
      
      gridProdutos.datasource.filter = conditions;
      gridProdutos.datasource.get();
      this.cardProdutoVisible();
    } else {
      this.selectedTiporetirada = `${component.currentRow.CDTIPORETI} - ${component.currentRow.NMTIPORETI}`;
      this.closeCardDetail();
    }
  }
  
  private cardProdutoVisible() {
    const card = Metadata.getInstance<Card>('detailProdutosCard');
    
    try {
      if (!this.gridEditController.newRow) {
        card.isVisible = true;
        this.detailIsVisible = true;
      }
    } catch (Error) {
      card.isVisible = false;
      this.detailIsVisible = false;
    }
  }
  
  public saveTipoRetirada() {
    const formTipoRetirada = Metadata.getInstance<Form>('formTipoRetirada');
    const gridTipoRetirada: GridEditable = Metadata.getInstance('gridTipoRetirada');
    const editedRows: IDictionary = gridTipoRetirada.getEditedRows();
    const datasourceName = 'tpRetirada';
    
    if (formTipoRetirada.validate() && editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        row.CDTIPORETI = row.CDTIPORETI ? row.CDTIPORETI.padStart(2, '0') : '';
        row.NMTIPORETI = row.NMTIPORETI ? row.NMTIPORETI : '';
        row.CDCONTCTBL = row.CDCONTCTBL ? row.CDCONTCTBL : '';
        row.CDHISTPADR = row.CDHISTPADR ? row.CDHISTPADR : '';
        row.CDFUNCUSTO = row.CDFUNCUSTO ? row.CDFUNCUSTO : '';
        row.IDTIPOMOVI = row.IDTIPOMOVI ? row.IDTIPOMOVI : 'A';
      });
      const gridProdutos: GridEditable = Metadata.getInstance('gridProdutos');

      this.crudController.saveRows(editedRows, datasourceName, gridTipoRetirada).then(() => {
        this.gridEditController.defaultState(gridTipoRetirada);
        gridProdutos.datasource.get();
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  public saveProdutos() {
    const gridDetail = Metadata.getInstance<GridEditable>('gridProdutos');
    const editedRows: IDictionary = gridDetail.getEditedRows();
    
    if (editedRows.length > 0) {
      const cdtiporeti = this.selectedTiporetirada.substring(0, 2);
      
      editedRows.forEach((row: IDictionary) => {
        const oldElement = gridDetail.datasource.data.find((el) => el.NMPRODUTO === row.NMPRODUTO);
        row.CDTIPORETI = cdtiporeti;
        row.OLDCDPRODRETI = oldElement?.CDPRODRETI;
      });
      
      Http.post('/saveProdutosTpRetirada', { requestType: 'Row', row: editedRows }).then((response: IDictionary) => {
        if (response) {
          this.gridEditController.defaultState(gridDetail);
          this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        }
      }).catch((error: string) => {
        if (error.includes('ORA-00001')) this.utilities.showAlert('alert', 'pme::impossivelAdicionar', 'error');
        else this.utilities.showAlert('alert', `pme::${error}`, 'error');
      }).finally(() => {
        //
      });
    }
  }
  
  public deleteProdutos() {
    const gridDetail = Metadata.getInstance<GridEditable>('gridProdutos');
    this.checkedRowsProdutos = gridDetail.selectedRows;
    
    if (this.checkedRowsProdutos.length > 0) {
      const cdtiporeti = this.selectedTiporetirada.substring(0, 2);
      
      this.checkedRowsProdutos.forEach((row: IDictionary) => {
        row.CDTIPORETI = cdtiporeti;
      });
      
      DialogService.show({
        name: 'excludeGridPratosDetailAction',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yesDeleteProdutos',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.callDeleteProdutos(gridDetail) },
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  private callDeleteProdutos(gridDetail: GridEditable) {
    Http.post('/deleteProdutosTipoRetirada', { requestType: 'Row', row: this.checkedRowsProdutos }).then(() => {
      gridDetail.cancelEditedRows();
      gridDetail.datasource.get();
      gridDetail.selectedRows = [];
      
      this.utilities.showAlert('alert', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      gridDetail.selectedRows = [];
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      gridDetail.selectedRows = [];
      DialogService.hide();
    });
  }
}
