import {
  Form, Select, Grid,
} from '@zeedhi/common';
import {
  Loader, Metadata, IEventParam, Config,
} from '@zeedhi/core';
import { PmeMask } from '@/config';

export class Pla01300AcoesPratoController {
  private appController = Loader.getInstance('AppController');

  private utilities = Loader.getInstance('Utilities');

  public prato = Loader.getInstance('Pla01300PratoController');
  
  public setFilterPrato({ component }: IEventParam<Select>) {
    const nivel: any = (Config.masks.plateMask as PmeMask).nivelMax;
    const unidade = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM.CDFILIAL;

    component.datasource.filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade,
      },
      {
        name: 'CDNVPRATO',
        operator: '=',
        value: nivel,
      },
    ];
    component.datasource.get();
  }
  
  public changeRecipeAcoesPrato({ component }: IEventParam<Select>) {
    const filial = Metadata.getInstance<Select>('UNIDADE');
    const cardapio = Metadata.getInstance<Select>('TIPOCARD');
    const servico = Metadata.getInstance<Select>('SERVICO');

    if (component.value === 'L') {
      filial.disabled = false;
      filial.addValidation('required', { message: 'pme::campoObrigatorio' });
      filial.setValue('');
      servico.disabled = true;
      servico.removeValidation('required');
      servico.setValue('');
      cardapio.disabled = true;
      cardapio.removeValidation('required');
      cardapio.setValue('');
    } else if (component.value === 'T') {
      filial.disabled = true;
      filial.removeValidation('required');
      filial.setValue('');
      servico.disabled = true;
      servico.removeValidation('required');
      servico.setValue('');
      cardapio.addValidation('required', { message: 'pme::campoObrigatorio' });
      cardapio.disabled = false;
    } else if (component.value === 'S') {
      filial.disabled = false;
      servico.disabled = false;
      cardapio.disabled = true;
      filial.addValidation('required', { message: 'pme::campoObrigatorio' });
      servico.addValidation('required', { message: 'pme::campoObrigatorio' });
      cardapio.removeValidation('required');
      cardapio.setValue('');
    } else if (component.value === 'P') {
      // fazer verificacao pro opLogado
      filial.removeValidation('required');
      servico.removeValidation('required');
      cardapio.removeValidation('required');
      filial.disabled = true;
      servico.disabled = true;
      cardapio.disabled = true;
      filial.setValue('');
      servico.setValue('');
      cardapio.setValue('');
      // this.setFilterAcaoPrato();
    }
    this.verificaPreenchimentoCampos();
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const servico = Metadata.getInstance<Select>('SERVICO');
    this.utilities.clearPeriodoParams();

    if (component.name === 'TIPOCARD' || component.name === 'GRUPFILI') {
      servico.setValue('');
      unidade.setValue('');
    } else if (component.name === 'UNIDADE' || component.name === 'GRUPOSERVICO') {
      servico.setValue('');
    } else unidade.setValue('');
  }
  
  public verificaPreenchimentoCampos() {
    const grid: Grid = Metadata.getInstance('gridPrato');
    const form: Form = Metadata.getInstance('formAcoesPrato');
    const filial = form.getChildInstance<Select>('UNIDADE');
    const cardapio = form.getChildInstance<Select>('TIPOCARD');
    const servico = form.getChildInstance<Select>('SERVICO');
    const row = form.value;
    
    if (((row.TPRECEITA === 'L' || row.TPRECEITA1 === 'L') && filial.value)
        || ((row.TPRECEITA === 'T' || row.TPRECEITA1 === 'T') && cardapio.value)
        || ((row.TPRECEITA === 'S' || row.TPRECEITA1 === 'S') && servico.value)
        || (row.TPRECEITA === 'P' && row.TPRECEITA1 === 'P')
    ) {
      this.setFilterAcaoPrato();
    } else grid.datasource.data = [];
  }
  
  public changeUnityServ() {
    const grid: Grid = Metadata.getInstance('gridPrato');
    const form: Form = Metadata.getInstance('formAcoesPrato');
    const row = form.value;
    
    if (row.TPRECEITA === 'S' || row.TPRECEITA1 === 'S') {
      const serv = Metadata.getInstance<Select>('SERVICO');
      serv.setValue('');
      grid.datasource.data = [];
    }
    this.verificaPreenchimentoCampos();
  }
  
  public setFilterAcaoPrato() {
    const form: Form = Metadata.getInstance('formAcoesPrato');
    const row = form.value;
    const unidade = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM.CDFILIAL;
    const nivel: any = (Config.masks.plateMask as PmeMask).nivelMax;
    const grid: Grid = Metadata.getInstance('gridPrato');
    
    if (this.utilities.returnPrat(row.PRATOINICIAL, row.PRATOFINAL) && row.PRATOFINAL !== null) {
      this.utilities.showAlert('Alert', 'pme::pratoIniMaior', 'error');
      row.PRATOFINAL = null;
    } else if (unidade) {
      const conditions = [
        {
          name: 'TMNIVEIS',
          operator: '=',
          value: nivel,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: unidade,
        },
        {
          name: 'CDPRATOINI',
          operator: '=',
          value: row.PRATOINICIAL,
        },
        {
          name: 'CDPRATOFIN',
          operator: '=',
          value: row.PRATOFINAL,
        },
        {
          name: 'TPRECE',
          operator: '=',
          value: row.TPRECEITA !== null ? row.TPRECEITA : row.TPRECEITA1,
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: row.TIPOCARD ? row.TIPOCARD : null,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: row.UNIDADE !== undefined ? row.UNIDADE : 'T',
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: row.SERVICO ? row.SERVICO : 'X',
        },
      ];
      grid.datasource.filter = conditions;
      grid.datasource.get();
    } else grid.datasource.data = [];
  }
  
  // verificar dados
  public setFilterService({ component }: IEventParam<Select>) {
    const form: Form = Metadata.getInstance('formAcoesPrato');
    const row = form.value;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: row.UNIDADE,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: localStorage.getItem('PRODUCT_ID') === '4875' ? '' : 'N',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
}
